import tkinter as tk
import os
import requests
import configparser

class ZynithWriter:
    def __init__(self):
        self.window = tk.Tk()
        self.window.title("ZynithWriter")
        self.window.geometry("600x800")

        self.api_key = self.load_api_key()

        self.api_key_label = tk.Label(self.window, text="Please enter your API key:", anchor="w")
        self.api_key_label.pack(fill="x", pady=10, padx=10)
        
        self.api_key_entry = tk.Entry(self.window, width=int(0.6*self.window.winfo_screenwidth()))
        self.api_key_entry.insert(0, self.api_key)
        self.api_key_entry.pack(pady=10, padx=10)

        if self.api_key:
            self.notification_label = tk.Label(self.window, text="Is this correct?", anchor="w")
        else:
            self.notification_label = tk.Label(self.window, text="", anchor="w")
        self.notification_label.pack(fill="x", pady=10, padx=10)

        self.go_create_button = tk.Button(self.window, text="Go Create", command=self.go_to_content_generator)
        self.go_create_button.pack(pady=(21, 21), padx=10)


    def load_api_key(self):
        config = configparser.ConfigParser()
        config_file = os.path.join(os.path.dirname(os.path.abspath(__file__)), "zynithwriter_config.ini")
        if os.path.exists(config_file):
            config.read(config_file)
            if "ZynithWriter" in config and "APIKey" in config["ZynithWriter"]:
                return config["ZynithWriter"]["APIKey"]
        return ""

    def save_api_key_to_config(self, api_key):
        config = configparser.ConfigParser()
        config_file = os.path.join(os.path.dirname(os.path.abspath(__file__)), "zynithwriter_config.ini")
        config["ZynithWriter"] = {"APIKey": api_key}
        with open(config_file, "w") as configfile:
            config.write(configfile)

    def go_to_content_generator(self):
        self.api_key = self.api_key_entry.get()
        if self.api_key and len(self.api_key.strip()) > 0:
            self.save_api_key_to_config(self.api_key)
            self.notification_label.config(text="API key stored successfully.")
            self.window.destroy()
            ContentGenerator(self.api_key)
        else:
            self.notification_label.config(text="Please enter a valid API key.")

    def run(self):
        self.window.mainloop()

class ContentGenerator:
    def __init__(self, api_key):
        self.api_key = api_key
        self.window = tk.Tk()
        self.window.title("ZynithWriter - Content Generator")
        self.window.geometry("600x800")

        self.h1_label = tk.Label(self.window, text="H1 Heading:", anchor="w")
        self.h1_label.pack(fill="x", pady=10, padx=10)
        self.h1_entry = tk.Entry(self.window, width=int(0.6*self.window.winfo_screenwidth()))
        self.h1_entry.pack(pady=10, padx=10)

        self.h2_label = tk.Label(self.window, text="H2 Headings (comma-separated):", anchor="w")
        self.h2_label.pack(fill="x", pady=10, padx=10)
        self.h2_entry = tk.Entry(self.window, width=int(0.6*self.window.winfo_screenwidth()))
        self.h2_entry.pack(pady=10, padx=10)

        self.faq_label = tk.Label(self.window, text="FAQs (comma-separated):", anchor="w")
        self.faq_label.pack(fill="x", pady=10, padx=10)
        self.faq_entry = tk.Entry(self.window, width=int(0.6*self.window.winfo_screenwidth()))
        self.faq_entry.pack(pady=10, padx=10)


        # Add checkboxes for formality levels
        self.formality_label = tk.Label(self.window, text="Formality Level:", anchor="w")
        self.formality_label.pack(fill="x", pady=10, padx=10)

        self.formality_var = tk.StringVar(value="Conversational")  # Default value
        self.formality_checkboxes = [
            ("Conversational", "Conversational"),
            ("Informal", "Informal"),
            ("Semi Formal", "SemiFormal"),
            ("Formal", "Formal"),
        ]

        for text, value in self.formality_checkboxes:
            checkbox = tk.Radiobutton(self.window, text=text, variable=self.formality_var, value=value)
            checkbox.pack(anchor="w", padx=20)

        # Add checkboxes for grade levels
        self.grade_label = tk.Label(self.window, text="Grade Level:", anchor="w")
        self.grade_label.pack(fill="x", pady=10, padx=10)

        self.grade_var = tk.StringVar(value="Elementary")
        self.grade_checkboxes = [
            ("Elementary", "Elementary"),
            ("Jr High", "JrHigh"),
            ("High School", "HighSchool"),
            ("Undergraduate", "Undergraduate"),
            ("Graduate", "Graduate"),
        ]

        for text, value in self.grade_checkboxes:
            checkbox = tk.Radiobutton(self.window, text=text, variable=self.grade_var, value=value)
            checkbox.pack(anchor="w", padx=20)

        self.filename_label = tk.Label(self.window, text="Output Filename (without extension):", anchor="w")
        self.filename_label.pack(fill="x", pady=10, padx=10)
        self.filename_entry = tk.Entry(self.window, width=int(0.6*self.window.winfo_screenwidth()))
        self.filename_entry.pack(pady=10, padx=10)

        self.generate_button = tk.Button(self.window, text="Generate Article and Save", command=self.generate_article_and_save)
        self.generate_button.pack(pady=(21, 21), padx=10)

        self.result_label = tk.Label(self.window, text="", anchor="w")
        self.result_label.pack(fill="x", pady=10, padx=10)

    def generate_content(self, prompt, api_key):
        url = "https://api.openai.com/v1/engines/text-davinci-003/completions"
        headers = {
            "Content-Type": "application/json",
            "Authorization": f"Bearer {api_key}",
        }
        data = {
            "prompt": prompt,
            "temperature": 0.7,
            "max_tokens": 2000,
        }
        response = requests.post(url, json=data, headers=headers)
        if response.status_code != 200:
            print(f"Error: {response.status_code} - {response.text}")
            return ""
        try:
            response_json = response.json()
            content = response_json["choices"][0]["text"]
            return content
        except KeyError as e:
            print(f"KeyError: {e}")
            return ""

    def generate_article(self, h1, h2_list):
        formality_level = self.formality_var.get()  # Get the selected formality level
        grade_level = self.grade_var.get()  # Get the selected grade level

        # Modify prompts based on the selected formality and grade levels
        intro_prompt = f"In the formality level of {formality_level.lower()} Please write an introduction about the following topics meant to be understood and comprehended by {grade_level.lower()} students: '{', '.join(h2_list)}'. Do not use the word students in the text as it is not directly for students."
        content = f"{h1}\n{self.generate_content(intro_prompt, self.api_key)}\n\n"

        for index, h2 in enumerate(h2_list):
            h2_content = self.generate_content(f"In the formality level of {formality_level.lower()}, Please write about '{h2}' in comprehensive detail meant to be understood comprehended by {grade_level.lower()} students. Start with an appealing hook in the first sentence. Include relevant statistics and case studies as needed. Do not use the word students in the text as it is not directly for students.", self.api_key)
            content += f"\n{h2}\n{h2_content}\n\n"

        faq_input = self.faq_entry.get()
        faq_list = [faq.strip() for faq in faq_input.split(",")]
        if faq_list:
            content += "\nFAQs\n\n"
            for faq in faq_list:
                answer = self.generate_content(f"In the formality level of {formality_level.lower()}, Answer the question in long-form content meant to be read and understood by {grade_level.lower()} students. The first sentence should directly answer the query in a complete sentence, followed by additional information: '{faq}'. Do not use the word students in the text as it is not directly for students.", self.api_key)
                content += f"\nQ: {faq}\nA: {answer.strip()}\n"

        conclusion_prompt = f"In the formality level of {formality_level.lower()}, Please write a comprehensive conclusion summarizing the detailed information discussed about '{', '.join(h2_list)}' topics meant to be understood and comprehended by {grade_level.lower()} students, ensuring to use longer, compounded sentences and a final thought about the information. Do not use the word students in the text as it is not directly for students."
        content += f"\n\nConclusion\n{self.generate_content(conclusion_prompt, self.api_key)}"

        return content

    def save_to_txt_file(self, article_text, filename):
        script_directory = os.path.dirname(os.path.abspath(__file__))
        if not filename.endswith(".txt"):
            filename += ".txt"
        file_path = os.path.join(script_directory, filename)
        with open(file_path, "w") as file:
            file.write(article_text)

    def generate_article_and_save(self):
        h1 = self.h1_entry.get()
        h2_input = self.h2_entry.get()
        h2_list = [h.strip() for h in h2_input.split(",")]

        article_text = self.generate_article(h1, h2_list)
        filename = self.filename_entry.get()
        if article_text:
            self.save_to_txt_file(article_text, filename)
            self.result_label.config(text=f"Article generated and saved to {filename}")
        else:
            self.result_label.config(text="Error generating the article.")

    def run(self):
        self.window.mainloop()

if __name__ == "__main__":
    ZynithWriter().run()
